import pytest


@pytest.mark.usefixtures("receptor_mesh_mesh1")
class TestReceptorCtlConnection:
    def test_connect_to_service(self, default_receptor_controller_unix):
        node1_controller = default_receptor_controller_unix
        node1_controller.connect_to_service("node2", "control", "")
        node1_controller.handshake()
        status = node1_controller.simple_command("status")
        node1_controller.close()
        assert status["NodeID"] == "node2"

    def test_simple_command(self, default_receptor_controller_unix):
        node1_controller = default_receptor_controller_unix
        status = node1_controller.simple_command("status")
        node1_controller.close()
        assert not (
            set(
                [
                    "Advertisements",
                    "Connections",
                    "KnownConnectionCosts",
                    "NodeID",
                    "RoutingTable",
                ]
            )
            - status.keys()
        )

    def test_simple_command_fail(self, default_receptor_controller_unix):
        node1_controller = default_receptor_controller_unix
        with pytest.raises(RuntimeError):
            node1_controller.simple_command("doesnotexist")
        node1_controller.close()

    def test_tcp_control_service(self, default_receptor_controller_tcp):
        node1_controller = default_receptor_controller_tcp
        status = node1_controller.simple_command("status")
        node1_controller.close()
        assert not (
            set(
                [
                    "Advertisements",
                    "Connections",
                    "KnownConnectionCosts",
                    "NodeID",
                    "RoutingTable",
                ]
            )
            - status.keys()
        )

    def test_tcp_control_service_tls(self, default_receptor_controller_tcp_tls):
        node1_controller = default_receptor_controller_tcp_tls
        status = node1_controller.simple_command("status")
        node1_controller.close()
        assert not (
            set(
                [
                    "Advertisements",
                    "Connections",
                    "KnownConnectionCosts",
                    "NodeID",
                    "RoutingTable",
                ]
            )
            - status.keys()
        )



from receptorctl import cli as commands

# The goal is to write tests following the click documentation:
# https://click.palletsprojects.com/en/8.0.x/testing/

import pytest


@pytest.mark.usefixtures("receptor_mesh_mesh1")
class TestCLI:
    def test_cli_cmd_status(self, invoke_as_json):
        result, json_output = invoke_as_json(commands.status, [])
        assert result.exit_code == 0
        assert set(
            [
                "Advertisements",
                "Connections",
                "KnownConnectionCosts",
                "NodeID",
                "RoutingTable",
                "SystemCPUCount",
                "SystemMemoryMiB",
                "Version",
            ]
        ) == set(json_output.keys()), "The command returned unexpected keys from json output"



from receptorctl import cli as commands

# The goal is to write tests following the click documentation:
# https://click.palletsprojects.com/en/8.0.x/testing/

import pytest
import time


@pytest.mark.usefixtures("receptor_mesh_access_control")
class TestMeshFirewall:
    def test_work_unsigned(self, invoke, receptor_nodes):
        """Run a unsigned work-command

        Steps:
        1. Create node1 with a unsigned work-command
        2. Create node2
        3. Run from node2 a unsigned work-command to node1
        4. Expect to be accepted
        """

        # Run an unsigned command
        result = invoke(
            commands.work,
            "submit unsigned-echo --node node1 --no-payload".split(),
        )
        work_unit_id = result.stdout.split("Unit ID: ")[-1].replace("\n", "")

        time.sleep(5)
        assert result.exit_code == 0

        # Release unsigned work
        result = invoke(commands.work, f"release {work_unit_id}".split())
        time.sleep(5)

        assert result.exit_code == 0

    # DISABLE UNTIL THE FIX BEING IMPLEMENTED
    #
    # def test_work_signed_expect_block(self, invoke, receptor_nodes):
    #     """Run a signed work-command without the right key
    #     and expect to be blocked.

    #     Steps:
    #     1. Create node1 with a signed work-command
    #     2. Create node2
    #     3. Run from node2 a signed work-command to node1
    #     4. Expect to be blocked
    #     """
    #     # Run an unsigned command
    #     result = invoke(
    #         commands.work, "submit signed-echo --node node1 --no-payload".split()
    #     )
    #     work_unit_id = result.stdout.split("Unit ID: ")[-1].replace("\n", "")

    #     time.sleep(5)
    #     assert work_unit_id, "Work unit ID should not be empty"
    #     assert result.exit_code != 0, "Work signed run should fail, but it worked"

    #     # Release unsigned work
    #     result = invoke(commands.work, f"release {work_unit_id}".split())
    #     assert result.exit_code == 0, "Work release failed"
